#ifndef DECODER_H
#define DECODER_H 1

#include "dtmf.h"               /* defines 'byte' */

/**
 * The result of the scan of the audio input, giving the decoded message,
 * its length, and the location of the message start within the audio.
 */
typedef struct {
    byte* decoded;
    size_t allocated;
    size_t decoded_length;
    float start_offset;         /* seconds */
    float audio_length;         /* seconds */
} DTMFX_Result;

/**
 * Parameters controlling the filtering process.
 * These are not extensively documented, deliberately,
 * since they may change without (much) notice.
 */
typedef struct {
    float filter_length;        /* length of filter in seconds */
    float background_mult;      /* multiplier of default noise floor */
    float min_tone_length;      /* minimum tone length */
    int collect_all_digits_p;   /* if true, collect all digits, even
                                   if they fail consistency checks
                                   (eg for gaps between them) */
} DTMFX_FilterParameters;

/**
 * A typedef giving a number of valid file types.
 * FILE_UNKNOWN is a dummy/sentinel value.
 */
typedef enum { FILE_UNKNOWN, FILE_MP3, FILE_WAV } DTMFX_FileFormat;

DTMFX_Result* dtmfx_decode_bytes_from_audio(const char* filename,
                                            DTMFX_FilterParameters*,
                                            DTMFX_FileFormat);
void dtmfx_initialise_filter_parameters(DTMFX_FilterParameters*);
void dtmfx_decoder_free_result(DTMFX_Result*);

#if DTMFX_IMPLEMENTATION

typedef float Sample;

int process_frame(void *data,
                  Sample* samples,
                  int samplerate,
                  int nsamples);

#endif /* DTMFX_IMPLEMENTATION */

#endif /* DECODER_H */
