/*
 * This file contains implementations for handle_wav_file_* for multiple bit widths.
 *
 * The including file must define DTMF_SOUND_WIDTH=1, 2, 3, or 4
 */

#if DTMF_SOUND_WIDTH == 1
#define HANDLE_WAV_FILE      handle_wav_file_1
#define SAMPLE_TYPE          uint8_t
#elif DTMF_SOUND_WIDTH == 2
#define HANDLE_WAV_FILE      handle_wav_file_2
#define SAMPLE_TYPE          int16_t
#elif DTMF_SOUND_WIDTH == 3
#define HANDLE_WAV_FILE      handle_wav_file_3
#define SAMPLE_TYPE          int32_t
#elif DTMF_SOUND_WIDTH == 4
#define HANDLE_WAV_FILE      handle_wav_file_4
#define SAMPLE_TYPE          int32_t
#else
#error "Impossible value of DTMF_SOUND_WIDTH"
#endif

#define FRAME_SIZE      1024

static int HANDLE_WAV_FILE(cbyte *buf, cbyte* endbuf, FmtCkPCMHeader* fmt, void* analyser_state)
{
    int nchannels = fmt->channels;
    int samples_per_sec = fmt->samples_per_sec;
    Sample samples[FRAME_SIZE];
    int sample_no;
#if DTMF_SOUND_WIDTH == 3
    int8_t* p;
    int8_t* endp;
#else    
    SAMPLE_TYPE* p;
    SAMPLE_TYPE* endp;
#endif
    int finished_p = 0;
    
    if (nchannels != 1) {
        error_exit("Can only handle one channel right now, not %d", nchannels);
    }

#if DTMF_SOUND_WIDTH == 3
    p = (int8_t*)buf;
    endp = (int8_t*)endbuf;
#else
    p = (SAMPLE_TYPE*)buf;
    endp = (SAMPLE_TYPE*)endbuf;
#endif

    while (! finished_p) {
        for (sample_no=0; sample_no<FRAME_SIZE && p<endp; sample_no++) {
#if DTMF_SOUND_WIDTH == 1
            samples[sample_no] = *p - 64;
            p++;
#elif DTMF_SOUND_WIDTH == 3
            samples[sample_no] = ((p[2] * 256) + p[1]) * 256 + p[0];
            p += 3;
#else
            samples[sample_no] = *p;
            p++;
#endif
        }
        assert(p <= endp);
        process_frame(analyser_state,
                      samples,
                      samples_per_sec,
                      sample_no);
        if (p == endp) {
            finished_p = 1;
        }
    }
    return 0;
}

#undef FRAME_SIZE
#undef HANDLE_WAV_FILE
#undef SAMPLE_TYPE
