#! /bin/sh -

HEX2DTMF=../hex2dtmf

# Find a suitable MD5 program, and handle its peculiarities
for f in md5 md5sum; do
    m=`which $f`
    if test $? -eq 0; then
        case $f in
            md5) MD5run() { $m -r $1 | awk '{print $1}'; };;
            md5sum) MD5run() { $m $1 | awk '{print $1}'; };;
            *) echo "test-creating.sh: Unrecognised md5 hit! ($m)"; exit 1;;
        esac
        break
    fi
done

status=0

passes=
fails=

# Do the tests by creating a WAV file, and comparing the first 44
# bytes of the file with a known-good example in correct-*.  The first
# 44 bytes comprise the RIFF, WAVE, and 'fmt ' chunks, and the header
# of the 'data' chunk.
#
# It's possible to do a more thorough job, by doing a checksum of the
# entire output file.  However this seems too fragile, since we get
# slight differences between the outputs on different platforms.  I
# think these are probably due to minor maths differences, and
# therefore trivial.
testargs() {
    fn="temp$1.wav"
    rm -f "$fn" test-$1
    #echo $HEX2DTMF $1 123a456b789ce0fd
    $HEX2DTMF $1 123a456b789ce0fd >"$fn"
    head -c 44 "$fn" | cmp -s test-creating-good/correct"$1" - && success=: || success=false
    if $success; then
        passes=x$passes
        rm -f "$fn"
    else
        echo "Args $1: WAV header differs between $fn and correct$1"
        fails=x$fails
    fi
}

# The tests...
testargs -b8
testargs -b12
testargs -b20
testargs -b24
testargs -b32
testargs -bf

# Check creating JSON-encoded output
fn=temp-json.wav
$HEX2DTMF -b8 -fjson '[ 1, "x" ]' >$fn
head -c 44 $fn | cmp -s test-creating-good/correct-b8 - && success=: || success=false
if $success; then
    # check length of new file
    fs=`stat -f %z $fn`
    if test $fs = 25957; then success=:; else success=false; fi
fi
if $success; then
    passes=x$passes
    rm -f $fn
else
    echo "Creating JSON: WAV header differs between $fn and correct-b8, or file is the wrong size"
    fails=x$fails
fi

# Results
if test -z "$fails"; then
    npasses=`expr "$passes" : x\*`
    echo "Creating tests: ALL $npasses tests pass"
else
    nfails=`expr $fails : x\*`
    echo "$nfails tests failed"
    status=1
fi

exit $status
