#! /usr/bin/env python

import sys
import json
import subprocess
import os

# We have to normalise the path to the input filenames.
# (make distcheck does a VPATH build, so does Clever things with
# directories, which means we have to be specific about where things are.

dtmf2hex=os.path.abspath('../dtmf2hex')
testdir = os.path.abspath(os.path.dirname(sys.argv[0]))

if len(sys.argv) < 2:
    input_filename = 'test-reading.config'
else:
    input_filename = sys.argv[1]

# The subprocess.check_output function is the best way to interact
# with a subprocess, but it only appeared in Python 2.7, so only use
# it in that case.  The subprocess package only appeared in v2.4, but
# I'm disinclined to support versions that far back.
def subprocess_with_output(arglist, stdin_fd):
    if sys.hexversion > 0x02070000:
        try:
            output = subprocess.check_output(arglist,
                                             stdin=stdin_fd,
                                             stderr=subprocess.STDOUT)
            return (output, 0)
        except subprocess.CalledProcessError as e:
            return (None, e.returncode)
            #dtmf = { 'status': e.returncode, 'stderr': e.output }

    elif sys.hexversion > 0x02040000:
        p = subprocess.Popen(arglist, 
                             stdin=stdin_fd,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT)
        # Use p.communicate, since we won't be returning a lot of
        # data, so are in little danger of filling up a buffer.
        (stdoutdata, stderrdata) = p.communicate(None)
        return (stdoutdata, p.returncode)

    else:
        sys.stderr.write('test-reading.py: this test script requires at least Python v2.4\n')
        sys.exit(1)

nsuccess = 0
nfail = 0

with open(os.path.join(testdir,input_filename)) as f:
    tests = json.load(f)

    for t in tests:
        if 'file' in t:
            testname = t['file']
        elif 'stdin' in t:
            testname = 'stdin:' + t['stdin']
        else:
            testname = '???'
        try:
            arglist=[dtmf2hex]
            arglist.extend(t['opts'] if 'opts' in t else ['-tjstring'])

            # Source of input
            input_fd=None
            if 'file' in t:
                arglist.append(os.path.join(testdir,t['file']))
            elif 'stdin' in t:
                input_fd = open(t['stdin'])

            (dtmf_output, status) = subprocess_with_output(arglist, input_fd)

            if status == 0:
                if 'json' in t:
                    dtmf = json.loads(dtmf_output)
                else:
                    dtmf = { 'string': dtmf_output.strip() }
            else:
                # error return
                dtmf = { 'status': status, 'stderr': dtmf_output }

        except ValueError as e:
            print "Test %s failed: can't parse JSON: %s" % (testname, dtmf_output)

        if 'status' in t:       # check return status
            if 'status' in dtmf and t['status'] == dtmf['status']:
                nsuccess = nsuccess + 1
                if t['status'] != 0: # (expected) error, so don't do any more checks
                    continue

        if 'status' in dtmf and dtmf['status'] != 0:    # there was an error exit status
            print "Test %s failed:\n    got unexpected exit status %d (%s)" \
                % (testname, dtmf['status'], dtmf['stderr'])
            nfail = nfail + 1
            continue

        if 'json' in t:
            exp = t['json']
            # We simple test all of the fields.
            if t['json'] == dtmf:
                nsuccess = nsuccess + 1
            else:
                print "Test %s:\n    expected: %s\n      actual: %s" % (testname, str(dtmf), str(t['json']))
                nfail = nfail + 1
            # An alternative is to test just the output string.
            # Testing all of the fields can give spurious
            # fails since I'm still tweaking the algorithms in ways
            # which change the times when tones start.
            # if exp['string'] == dtmf['string']:
            #     nsuccess = nsuccess + 1
            # elif not ('json' in t):
            #     print "Test %s: no expected results!" % testname
            #     nfail = nfail + 1
            # else:
            #     print "Test %s failed:\n    got:      %s / %.3f / %.3f / %.3f\n    expected: %s / %.3f / %.3f / %.3f" % \
            #         (testname,
            #          dtmf['string'],dtmf['start_offset'],dtmf['end_offset'],dtmf['audio_length'],
            #          exp['string'],exp['start_offset'],exp['end_offset'],exp['audio_length'])
            #     nfail = nfail + 1
        elif 'string' in t:
            if t['string'] == dtmf['string']:
                nsuccess = nsuccess + 1
            else:
                print "Test %s failed:\n    got:      %s\n    expected: %s" % (testname,
                                                                               dtmf['string'],
                                                                               t['string'])
                nfail = nfail + 1
        else:
            # This probably shouldn't happen
            print "Test %s failed (I think): expected %s, but got no string response" % (testname, dtmf['string'])

if nfail == 0:
    print "Reading tests: ALL %d tests pass" % nsuccess
else:
    print "Reading tests: %d/%d tests failed" % (nfail, nfail+nsuccess)
    sys.exit(1)
