Jason
=====

A JSON to ASN.1/DER encoder/decoder, in C.

The `libjason` library is able to parse JSON, and encode it to ASN.1
using the Distinguished Encoding Rules (DER) of ITU-T X.690.

The library parses JSON as specified in [RFC 4627][], with no
extensions.  However the string parser cannot at present handle
Unicode strings (which also means it doesn't respect the JSON `\uXXXX`
escape); all strings must be ASCII. 

The DER *de*coder can handle some BER-encoded objects, but not all,
and it doesn't attempt to handle ASN.1 types other than the subset
which it generates.

In that encoding, JSON integers, floats, booleans, and null, are
encoded to the DER analogues.  A JSON array is encoded to a DER
`SEQUENCE`, an object to a `SET OF` two-element sequences, and strings
to DER `ia5string` objects.

DER is specified in ITU-T Rec. X.690, as part of the ASN/1 suite of
standards.  See the [ITU-T recommendation][] and [Wikipedia][], and
Burton S. Kaliski Jr., *A Layman's Guide to a Subset of ASN.1, BER,
and DER* (1993).


Building
--------

Building from a distribution:

    ./configure
    make
    make check

If building from the repository, then this build sequence must be
preceded by `./bootstrap`, and the GNU autotools must be installed.

The build depends on `flex` and `bison` being in the path.
There are no other dependencies.


Utility program
---------------

As well as a library (documented below), this kit includes a program
`json2der` which encodes and decodes JSON to DER on the command-line.
See the man-page `json2der.1` for details.


Interface
---------

The interface is still somewhat preliminary, and not documented very
elaborately.  However...

    #include <jason.h>

    JsonObject parse_json_string(const char* json_string);

Parse the JSON object given in the string.  If the string cannot be
parsed, then this returns `NULL`, and an explanation is available from
the function `const char* unsupported_message()`.  The returned object
should be freed by `free_json_object`.

    void free_json_object(JsonObject o);

Free the object.  Any allocated memory within the object is freed also.

    const char* print_json_object(JsonObject obj);

Serialise the object to a string.  The string must be subsequently
freed by the caller.

    uint8_t* get_der_encoding(JsonObject obj, size_t *len);

Get the DER-encoding of the object.  The object must not be freed by
the caller.

    char* bytes_to_string(uint8_t* b, size_t blen);

Utility method to serialise a byte sequence to a printable string of
hex digits.  The returned object must be freed by the caller.

    JsonObject parse_der_bytes(uint8_t*, size_t, size_t*);

Decode a sequence of bytes which are the DER encoding of a JSON
object, and return the object.  If the byte sequence
cannot be parsed, then this returns `NULL`, and an explanation is
available from the function `const char* unsupported_message()`.
The result should be freed by `free_json_object` when no longer required.  

    const char* jason_version_string(void);
    int jason_version_number(void);

Return information about the library version number.

Though this library is not intended as a general JSON parser, the
contents of the `JsonObject` structure should be fairly stable.  See
the `jason.h` header for the details.


Name
----

The name *Jason* is a fairly obvious amalgam of JSON and ASN.  JASN
would be a better name, but someone's had that idea before me, and
[that name's already taken][jasn] for a Java ASN parser.


Source code and licence information
-----------------------------------

The source code is available at <https://bitbucket.org/nxg/jason>.

The software is Copyright 2013 Norman Gray, and is released under the
terms of the [GNU General Public Licence, v3.0][]

The distribution includes the [`lcut` unit-testing framework][lcut],
which is Copyright 2005–2010 Tony Bai, and released under the terms of
the Apache Licence, Version 2.0.

> Jason is free software: you can redistribute it and/or modify
> it under the terms of the GNU General Public License as published by
> the Free Software Foundation, either version 3 of the License, or
> (at your option) any later version.
> 
> Jason is distributed in the hope that it will be useful,
> but WITHOUT ANY WARRANTY; without even the implied warranty of
> MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
> GNU General Public License for more details.
> 
> You should have received a copy of the GNU General Public License
> along with Jason.  If not, see <http://www.gnu.org/licenses/>.


Norman Gray  
http://nxg.me.uk


[ITU-T recommendation]: http://www.itu.int/rec/T-REC-X.690-200811-I/en
[Wikipedia]: http://en.wikipedia.org/wiki/X.690
[RFC 4627]: http://www.ietf.org/rfc/rfc4627.txt
[GNU General Public Licence, v3.0]: http://www.gnu.org/licenses/gpl-3.0.html
[jasn]: https://code.google.com/p/jasn/
[lcut]: https://github.com/bigwhite/lcut
