#ifndef JASON_H_DEFINED
#define JASON_H_DEFINED 1

#include <stdio.h>

#include <inttypes.h> /* includes stdint.h */


typedef intmax_t jsonint;
#define JSONFMT PRIiMAX

typedef enum {
    JSON_STRING,
    JSON_INTEGER,
    JSON_REAL,
    JSON_ARRAY,
    JSON_OBJECT,
    JSON_BOOLEAN,
    JSON_NULL,
    JSON_KVPAIR,                /* not exposed */
} JsonType;

struct json_object {
    JsonType type;
    uint8_t* der_encoding;
    size_t der_encoding_length;
    union {
        const char* s;
        jsonint i;
        double f;
        /* both arrays and objects are represented by having 'a'
           point to the first in a linked list of their contents, of
           whatever types */
        struct {
            struct json_object** p;
            size_t alloc;
            size_t len;
        } a;
        /* kv is a single key-value pair, which comprises the
           contents of an object */
        struct {
            const char* k;
            struct json_object* v;
        } kv;
        int b;
    } x;
};
typedef struct json_object* JsonObject;

JsonObject initialise_json_object(JsonObject, JsonType);
void free_json_object(JsonObject o);
const char* print_json_object(JsonObject obj);
uint8_t* get_der_encoding(JsonObject obj, size_t *len);
char* bytes_to_string(uint8_t* b, size_t blen);
JsonObject parse_json_string(const char* json_string);
JsonObject parse_der_bytes(uint8_t*, size_t, size_t*);
const char* unsupported_message();

const char* jason_version_string();
int jason_version_number();

#if JASON_IMPLEMENTATION

JsonObject make_string(const char*);
JsonObject make_integer(jsonint);
JsonObject make_real(double);
JsonObject make_boolean(int);
JsonObject make_null(void);
JsonObject make_array(JsonObject);
JsonObject make_object(JsonObject);
JsonObject make_kv(const char*, JsonObject);
JsonObject append_object(JsonObject, JsonObject);

#endif /* JASON_IMPLEMENTATION */

#endif /* JASON_H_DEFINED */
