/* a little parser for JSON */

%{
    #include <assert.h>
    #define JASON_IMPLEMENTATION 1
    #include "jason.h"

    int yylex(void);
    void yyerror (char const *msg);

    static JsonObject return_value;
    static void set_parsed_value(JsonObject);
    JsonObject get_parsed_value(void);
%}

%union {
    jsonint i;
    double f;
    char* str;
    struct json_object* o;
}

%token <str> STRING
%token <i> INTEGER
%token <f> REAL
%token COLON
%token COMMA
%token LEFTSQ
%token RIGHTSQ
%token LEFTCURLY
%token RIGHTCURLY
%token TRUE
%token FALSE
%token JNULL

%type <o> value array object list.of.value list.of.kv

%%
input: value { set_parsed_value($1); }

value: STRING { $$ = make_string($1); }
  | INTEGER { $$ = make_integer($1); }
  | REAL { $$ = make_real($1); }
  | TRUE { $$ = make_boolean(1); }
  | FALSE { $$ = make_boolean(0); }
  | JNULL { $$ = make_null(); }
  | object
  | array

array: LEFTSQ RIGHTSQ {
    $$ = make_array(NULL);
  }
  | LEFTSQ list.of.value RIGHTSQ {
    $$ = $2;
  }

list.of.value: value {
    $$ = make_array($1);
  }
  | list.of.value COMMA value {
    $$ = append_object($1, $3);
  }

object: LEFTCURLY RIGHTCURLY {
    $$ = make_object(NULL);
  }
  | LEFTCURLY list.of.kv RIGHTCURLY {
    $$ = $2;
  }

list.of.kv: STRING COLON value {
    $$ = make_object(make_kv($1, $3));
  }
  | list.of.kv COMMA STRING COLON value {
    $$ = append_object($1, make_kv($3, $5));
  }

%%
static void set_parsed_value(JsonObject o)
{
    return_value = o;
}
 
JsonObject get_parsed_value(void)
{
    return return_value;
}
 
