%{
    /* need this for the call to atof() below */
#include <math.h>
#include <assert.h>
#include <ctype.h>
#define JASON_IMPLEMENTATION 1
#include "jason.h"
#include "json-parse.tab.h"
#include "util.h"

//static char* copy_string(const char* s);
static StringBuilder stringbuilder;

#define ERR_MSG_LEN 64
static char err_msg[ERR_MSG_LEN];
char* lex_error_message(void)
{
    if (err_msg[0] == '\0') {
        return NULL;
    } else {
        return err_msg;
    }
}

static const char* parse_string_endp = NULL;
static const char* parse_string_p = NULL;
static const char* parse_string_buf = NULL;
int lex_setup(const char*);
void lex_release(void);

#define YY_INPUT(buf, result, max_size)                                 \
    {                                                                   \
        if (parse_string_p == NULL || parse_string_endp == NULL) {      \
            fprintf(stderr, "fatal: Bad call to parser: need to call lex_setup first\n"); \
            exit(1);                                                    \
        }                                                               \
        assert(parse_string_p != NULL && parse_string_endp != NULL);    \
        if (parse_string_p >= parse_string_endp) {                      \
            parse_string_p = parse_string_endp;                         \
            result = YY_NULL;                                           \
        } else {                                                        \
            int cpylen = (parse_string_endp - parse_string_p > max_size \
                          ? max_size                                    \
                          : parse_string_endp - parse_string_p);        \
            assert(buf != NULL);                                        \
            memcpy(buf, parse_string_p, cpylen);                        \
            parse_string_p += cpylen;                                   \
            result = cpylen;                                            \
        }                                                               \
        assert(parse_string_p <= parse_string_endp);                    \
    }
 

%}

DIGIT	[0-9]
WS	[[:space:]]*

%x string
%option noyywrap

%%

-?(0|[1-9][0-9]*)"."[0-9]+([eE][+-]?[0-9]+)?{WS}	yylval.f = strtod(yytext, NULL); return REAL;
-?(0|[1-9][0-9]*)("."[0-9]+)?([eE][+-]?[0-9]+){WS}	yylval.f = strtod(yytext, NULL); return REAL;


("-"|"+")?{DIGIT}+{WS}	yylval.i = strtoimax(yytext, NULL, 10); return INTEGER;

":"{WS}  		return COLON;

","{WS}			return COMMA;

\"			BEGIN(string); stringbuilder = make_append_buffer();
<string>[^\\"\n]*	append_to_buffer_s(stringbuilder, yytext);
<string>\\\"		append_to_buffer_c(stringbuilder, '\"');
<string>\\\\		append_to_buffer_c(stringbuilder, '\\');
<string>\\\/		append_to_buffer_c(stringbuilder, '/');
<string>\\b		append_to_buffer_c(stringbuilder, '\b');
<string>\\f		append_to_buffer_c(stringbuilder, '\f');
<string>\\n		append_to_buffer_c(stringbuilder, '\n');
<string>\\r		append_to_buffer_c(stringbuilder, '\r');
<string>\\t		append_to_buffer_c(stringbuilder, '\t');
<string>\\u		snprintf(err_msg, ERR_MSG_LEN, "don't support unicode yet\n"); return 0;
<string>\\.		snprintf(err_msg, ERR_MSG_LEN, "bogus escape '%s' in string\n", yytext); return 0;
<string>\n		snprintf(err_msg, ERR_MSG_LEN, "newline in string\n"); return 0;
<string>\"{WS}		{
        append_to_buffer_c(stringbuilder, '\0');
        yylval.str = (char*)stringbuilder->buf;
        free(stringbuilder);
        BEGIN(INITIAL);
        return STRING;
    }

"["{WS}			return LEFTSQ;
"]"{WS}			return RIGHTSQ;
"{"{WS}			return LEFTCURLY;
"}"{WS}			return RIGHTCURLY;

"true"{WS}		return TRUE;
"false"{WS}		return FALSE;
"null"{WS}		return JNULL;

.			snprintf(err_msg, ERR_MSG_LEN, "invalid character: %c", yytext[0]); return 0;

%%

#if 0
static char* copy_string(const char* s) 
{
    size_t slen = strlen(s);
    char* r;
    if ((r = (char*)malloc(slen+1)) == NULL) {
        fprintf(stderr, "Can't allocate %ld bytes for string\n", slen);
        exit(1);
    }
    strcpy(r, s);
    return r;
}
#endif
 
int lex_setup(const char* s) 
{
    int slen;                   /* length of buffer */
    
    YY_FLUSH_BUFFER;            /* flex-specific? */
    /* ensure we're in the start state (in case this function is
       called twice for some reason, such as showing the sequence of
       lexemes before the parse) */
    BEGIN(0);

    if (parse_string_buf != NULL) {
        // we've been called before -- forget that
        lex_release();
    }
    assert(parse_string_buf == NULL);

    /* Initialise/update the length of the copy_string buffer */
    //copy_string_init(s);

    // skip any leading whitespace (handling this in the grammar
    // complicates it disproportionately)
    while (*s != '\0' && isspace(*s)) {
        s++;
    }

    slen = strlen(s) + 1;
    if ((parse_string_buf = (const char*)malloc(slen)) == NULL) {
        fprintf(stderr, "Can't allocate memory for %d-byte string", slen);
        exit(1);                /* too drastic? */
    }
    assert(parse_string_buf != NULL);

    memcpy((void*)parse_string_buf, (const void*)s, slen);
    parse_string_p = parse_string_buf;
    parse_string_endp = parse_string_p + slen - 1; /* point to trailing '\0' */
    assert (*parse_string_endp == '\0');

    err_msg[0] = '\0';
    //invalid_character = '\0';

    // insert a fake call to unput, to suppress a "defined but not
    // used" warning
    while (0) {
        unput('x');
    }

    return 0;
}

void lex_release(void) 
{
    // lex_release releases the memory allocated in lex_setup.
    // Without this, we'd have a memory leak after repeated calls.
    // I spit on C.
    if (parse_string_buf != NULL) {
        free((void*)parse_string_buf);
    }
    parse_string_buf = parse_string_p = parse_string_endp = NULL;
    assert(parse_string_buf == NULL);

    return;
}
