/*
 * This file is part of the Jason library.
 * See <https://bitbucket.org/nxg/jason>
 *
 * Copyright 2013, Norman Gray <http://nxg.me.uk>
 *
 * It is distributed under the terms of the GNU General Public
 * Licence.  See the file README.me for details.
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
//#include <unistd.h>
//#include <fcntl.h>
#include <assert.h>
#include <string.h>
//#include <sys/stat.h>
//#include <sys/mman.h>

#include "util.h"

/*
 * Produce an error message, and exit with a non-zero status.
 * If the first character of 'fmt' is '!', then abort instead.
 */
void error_exit(const char* fmt, ...)
{
    va_list ap;
    int abort_p = 0;
    const char* actual_fmt = fmt;
    
    va_start(ap, fmt);

    if (*fmt == '!') {
        abort_p = 1;
        actual_fmt = &fmt[1];
    }
    
    fprintf(stderr, "libjason: ");
    vfprintf(stderr, actual_fmt, ap);
    if (abort_p) {
        fprintf(stderr, " [this shouldn't happen]");
    }
    fprintf(stderr, "\n");
    va_end(ap);

    if (abort_p) {
        abort();                /* flushes and closes streams */
    } else {
        exit(1);
    }
}

#if 0
const unsigned char* map_file(const char* filename, size_t* file_size)
{
    struct stat S;
    int fileno;
    void* mapped_file;

    if ((fileno = open(filename, O_RDONLY)) < 0) {
        error_exit("Can't open file %s to read", filename);
    }
    if (fstat(fileno, &S) == -1 || S.st_size == 0) {
        error_exit("Can't stat open file %s", filename);
    }
    mapped_file = mmap(0, S.st_size, PROT_READ, MAP_FILE|MAP_PRIVATE, fileno, 0);
    if (mapped_file == MAP_FAILED) {
        close(fileno);
        error_exit("Unable to map open file %s", filename);
    }

    if (file_size != NULL) {
        *file_size = S.st_size;
    }

    return (const unsigned char*)mapped_file;
}

void unmap_file(const unsigned char* mapped_file, size_t file_size)
{
    munmap((void*)mapped_file, file_size);
}
#endif

StringBuilder make_append_buffer()
{
    StringBuilder b;
    
    if ((b = (StringBuilder)malloc(sizeof(struct append_buffer))) == NULL) {
        error_exit("Can't allocate space for append_buffer");
    }
    b->alloc = 128;
    b->len = 0;
    if ((b->buf = (byte*)malloc(b->alloc)) == NULL) {
        error_exit("Can't allocate space for append_buffer");
    }
    return b;
}

void append_to_buffer_1(StringBuilder buf, byte b)
{
    if (buf->len == buf->alloc) {
        buf->alloc *= 2;
        if ((buf->buf = (byte*)realloc((void*)buf->buf, buf->alloc)) == NULL) {
            error_exit("Can't reallocate space for append_to_buffer_1");
        }
    }
    buf->buf[buf->len++] = b;
    assert(buf->len <= buf->alloc);
}

void append_to_buffer_n(StringBuilder buf, const byte* b, size_t blen)
{
    if (buf->len + blen >= buf->alloc) {
        while (buf->alloc < buf->len + blen) {
            buf->alloc *= 2;
        }
        if ((buf->buf = (byte*)realloc((void*)buf->buf, buf->alloc)) == NULL) {
            error_exit("Can't reallocate space for append_to_buffer_n");
        }
    }
    memcpy(&(buf->buf[buf->len]), b, blen);
    buf->len += blen;
    assert(buf->len <= buf->alloc);
}

void append_to_buffer_c(StringBuilder buf, char c)
{
    append_to_buffer_1(buf, (byte)c);
}

void append_to_buffer_s(StringBuilder buf, const char* s)
{
    size_t slen = strlen(s);
    append_to_buffer_n(buf, (byte*)s, slen);
}

